$!++
$!  FACILITY:
$!      GROUPS_CONVERT_V62
$!
$!  AUTHOR:
$!      Mark Martinec
$!      Copyright  1997
$!
$!  FUNCTIONALITY:
$!	Reads the file NEWS_ROOT:NEWS.GROUPS and copies it
$!	to NEWS_ROOT:NEW_NEWS.GROUPS, extending (or shrinking) the
$!	key0 field (extending to 80 bytes by default, see symbol
$!	grpnamelen_new).
$!
$!	In case of extending key0, the remaining fields are shifted right
$!	and the last bytes of the filler (which should be zero) are discarded.
$!	In case of shrinking key0 the newsgroups with names longer than
$!	grpnamelen_new are truncated which makes their articles unreachable.
$!
$!  BACKGROUND:
$!	Starting from V6.2.0, ANU-NEWS can handle newsgroup names longer
$!	than 49 characters.  ANU-NEWS in NNTP client mode can handle
$!	group names up to GRPNAMELEN-1 characters (currently 99)
$!	without modifications.  ANU-NEWS in local server mode now handles
$!	50-character groupnames (GRPNAMELEN_F) without modifications to files
$!	or source code. Before V6.2.0 the limit was 49;  the 50-char limit
$!	at the moment (June 1997) can handle all newsgroups
$!	in the big-8 Usenet hierarchies, but a couple of newsgroups
$!	(like in the 'alt' hierarchy) have longer names.
$!
$!	To handle newsgroup names longer than 50 characters
$!	in the local database you have to:
$!
$!	  - set constant GRPNAMELEN_F in file NEWSDEFINE.H
$!	    to a larger value (before V6.2.0 it was 50,
$!	    sugested new setting 80)
$!
$!	  - recompile (if executables are run directly from [.NEWS_DIST]
$!          shut down your news system first)
$!
$!	  - shut down your news system (server, feed, disable clients)
$!
$!	  - copy NEWS_ROOT:NEWS.GROUPS to a safe location (just in case)
$!
$!	  - run this command procedure (GROUPS_CONVERT_V62.COM)
$!	    (setting the symbol grpnamelen_new in this procedure to the
$!	    same value as GRPNAMELEN_F) and then replace the file
$!	    NEWS_ROOT:NEWS.GROUPS with a new file generated by this
$!	    procedure: NEWS_ROOT:NEW_NEWS.GROUPS
$!
$!	  - install new executables where users and site procedures expect them
$!
$!	  - restart news system (server, feed, enable clients)
$!
$!  VERSION:
$!	V6.2.0 10-Jun-1997  Mark Martinec  mark.martinec@ijs.si
$!	  - initial version
$!	V6.2.0 18-Jun-1997  Mark Pizzolato mark@infocomm.com
$!	  - minor correction to above instructions for conversion to 
$!	    new group name size.
$
$	on error then goto 199
$	say = "write sys$output"
$	on control_y then goto 199
$	RMS$_DUP = %X000184EC
$
$!!! ADJUST TO YOUR CONFIGURATION AND REQUIREMENTS:
$	grpnamelen_new = 80	! requested new key0 new size (grp_name)
$!	  this should match the constant GRPNAMELEN_F in NEWSDEFINE.H !
$!	  Lower limit is 50, upper limit is 95, suggested 80.
$
$	groups_file     = "NEWS_ROOT:NEWS.GROUPS"
$	new_groups_file = "NEWS_ROOT:NEW_NEWS.GROUPS"
$!!! NO CHANGES NEEDED BELOW THIS POINT
$
$	tmp_fdl_old = "0tmp.fdl"
$	tmp_fdl_new = "1tmp.fdl"
$! obtain FDL of the current file
$	say "Running ANALYZE/RMS/FDL ..."
$	analyze/rms/fdl/out='tmp_fdl_old' 'groups_file'
$
$	say "Analyzing FDL and preparing new FDL ..."
$	NEW_KEY0_SEG0_POSITION = 0
$	NEW_KEY0_SEG0_LENGTH   = grpnamelen_new
$	NEW_KEY1_SEG0_POSITION = grpnamelen_new
$	NEW_KEY1_SEG0_LENGTH   = 4
$
$	key = ""
$	section = ""
$	organization = ""
$	key0_seg0_length = ""
$	key0_seg0_position = ""
$	key1_seg0_length = ""
$	key1_seg0_position = ""
$
$! Parse FDL to get the current key0 size.
$! Make new FDL with the new (extended) key0 size.
$	open/read  fi 'tmp_fdl_old'
$	open/write fo 'tmp_fdl_new'
$a1:	read/end=a99 fi inp_line
$	  line = f$edit(inp_line,"TRIM,COMPRESS,UNCOMMENT")
$	  if line.eqs."" then goto a_next
$	  c1 = f$extr(0,1,inp_line)
$	  if c1.nes." " .and. c1.nes."	"
$	  then
$	    section = line
$	    key = f$elem(1," ",section)
$	    goto a_next
$	  endif
$	  line = f$edit(line,"UPCASE")
$	  if f$elem(0," ",section).eqs."FILE" then -
	    if f$elem(0," ",line).eqs."ORGANIZATION" then -
	      organization = f$elem(1," ",line)
$	  if f$elem(0," ",section).eqs."KEY"
$	  then
$	    option = f$elem(0," ",line)
$	    if option.eqs."SEG0_LENGTH" .or. option.eqs."SEG0_POSITION"
$	    then
$	      key'key'_'option' = f$elem(1," ",line)
$	      inp_line = f$fao("!_!AS!_!_!SL", -
	        f$elem(0," ",f$edit(inp_line,"TRIM,COMPRESS")), -
	        &NEW_KEY'key'_'option' )
$	    endif
$	  endif
$a_next:  write/symbol fo inp_line
$	  goto a1
$a99:	close fi
$	close fo
$	if organization.nes."INDEXED"             then goto bad1
$	if key0_seg0_position.ne.0                then goto bad2
$	if key1_seg0_position.ne.key0_seg0_length then goto bad2
$	if key1_seg0_length.ne.4                  then goto bad2
$	if key0_seg0_length.lt.50                 then goto bad2
$	if key0_seg0_length.gt.95                 then goto bad2
$	goto b0
$
$bad1:	say "File organization not indexed: ", organization
$	goto 199
$bad2:	say "Unexpected key size or position in the current groups file"
$	show symbol key0_seg0_position
$	show symbol key0_seg0_length
$	show symbol key1_seg0_position
$	show symbol key1_seg0_length
$	goto 199
$
$b0:	grpnamelen_old  = key0_seg0_length ! key0 size (grp_name, v6.1: 50)
$	if grpnamelen_old.eq.grpnamelen_new
$	then
$	  say "Nothing to do, key size equals required key size"
$	  goto 199
$	endif
$
$	fixed_part_size = 417	! useful data bytes beyond key0 (v6.1 and v6.2)
$	filler_size_old = 512 - fixed_part_size - grpnamelen_old
$	filler_size_new = 512 - fixed_part_size - grpnamelen_new
$	n_discarded = filler_size_old - filler_size_new
$	filler_old[filler_size_old*8,0] = 0 ! filler_size_old null characters
$	filler_new[filler_size_new*8,0] = 0 ! filler_size_new null characters
$	if n_discarded.ge.0
$	then
$	  discarded[n_discarded*8,0] = 0    ! abs(n_discarded) null characters
$	else
$	  discarded[-n_discarded*8,0] = 0   ! abs(n_discarded) null characters
$	endif
$	say "Copying ''groups_file' to ''new_groups_file'
$	if filler_size_old.gt.filler_size_new then -
	  say "  and extending key0 of each record from ''grpnamelen_old' to ''grpnamelen_new' ..."
$	if filler_size_old.lt.filler_size_new then -
	  say "  and shrinking key0 of each record from ''grpnamelen_old' to ''grpnamelen_new' ..."
$	say ""
$
$	open/read  g             'groups_file'
$	create/fdl='tmp_fdl_new' 'new_groups_file'
$	open/read/write  new_g   'new_groups_file'	! open existing file
$	recno = 0
$b1:	read/end=b99 g line
$	  recno = recno+1
$	  if recno/20*20.eq.recno then say recno
$	  grp_name_old = f$extr(0,grpnamelen_old,line)
$	  grp_rest     = f$extr(grpnamelen_old,fixed_part_size,line)
$	  grp_fill_old = f$extr(fixed_part_size,filler_size_old,line)
$	  if n_discarded.le.0
$	  then
$	    grp_name_new = f$extr(0,grpnamelen_new,grp_name_old)
$	    grp_name_lost = f$extr(grpnamelen_new,-n_discarded,grp_name_old)
$	    if grp_name_lost.nes.discarded
$	    then
	      say "RecNo: ", recno, ", group name longer than ", grpnamelen_new, " characters:"
$	      say "  ", grp_name_old
$	      say "  WARNING: truncated to ", grp_name_new
$	      say ""
$	    endif
$	  else
$	    ! last n_discarded chars of the filler
$	    grp_fill_lost = f$extr(fixed_part_size+filler_size_old-n_discarded,n_discarded,line)
$	    if grp_fill_old .nes. filler_old then -
	      say "RecNo: ", recno, ", filler at the end of the record not null"
$	    if grp_fill_lost.nes.discarded
$	    then
$	      say "  information in the last n_discarded characters will be lost:"
$	      show symbol grp_fill_lost
$	      say ""
$	    endif
$	    grp_name_new = grp_name_old
$	    grp_name_new[grpnamelen_new*8,0] = 0
$	  endif
$	  write/symbol/error=wrfail new_g grp_name_new,grp_rest,filler_new
$	  goto b1
$wrfail:  sts = $status
$	  if sts.ne.RMS$_DUP then goto 199
$	  say "Duplicate key, lost: ", grp_name_new
$	  goto b1
$b99:	say "Number of records converted: ", recno
$	say ""
$	say "Done.  File ", new_groups_file, " created."
$	say ""
$	say "Now it may be a good opportunity to optimize FDL of the"
$	say "new groups file ''new_groups_file' before renaming it"
$	say "to NEWS_ROOT:NEWS.GROUPS and restarting ANU-NEWS."
$	say ""
$	say "To do so run ANALYZE/RMS/FDL/OUT=GROUPS.FDL ''new_groups_file'"
$	say "then invoke optimization from EDIT/FDL/ANALY=GROUPS GROUPS"
$	say "and CONVERT/FDL=GROUPS ''new_groups_file' NEWS_ROOT:NEWS.GROUPS"
$	say ""
$	say "Before restarting ANU-NEWS make sure that constant GRPNAMELEN_F
$	say "in NEWSDEFINE.H is ''grpnamelen_new', recompile if necessary"
$	say "and replace all executables (ANU-NEWS itself (NEWS.EXE) as well as"
$	say "NNTP servers and NNTP support programs, such as NNTP_XMIT)."
$
$199:	if f$log("fi")   .nes."" then close fi
$	if f$log("fo")   .nes."" then close fo
$	if f$log("g")    .nes."" then close g
$	if f$log("new_g").nes."" then close new_g
