$ ver = 'f$ver("NO")'
$ set = ""
$ set symbol/scope=(nolocal,noglobal)
$
$! Run the given file through PGP, discarding output,
$! and check for a valid signature.
$! Return status and id (usually e-mail address) of the signer.
$!
$! p1 ... filename with the preprocessed (converted to the PGP format)
$!        control article to be checked for PGP signature.
$!        p1 may be empty - this case is treated as though
$!        PGP did not find a signature in the file (status: 1 or 11)
$! p2 ... hierarchy name - the last component of a newsgroup name
$!
$! Writes to SYS$OUTPUT the success status of the form: nnn key_id signer_id
$! where nnn is:
$!   000 (OK)   signature expected; good signature and correct signer id
$!   001 (bad)  signature expected; no signature found
$!   002 (bad)  signature expected; unknown signature
$!   003 (bad)  signature expected; bad signature
$!   004 (bad)  signature expected; good signature but wrong signer for the specified hierarchy
$!
$!   011 (OK)   no signature expected; no signature found
$!   012 (ok?)  no signature expected; unknown signature
$!   013 (bad?) no signature expected; bad signature
$!   014 (ok?)  no signature expected; good signature by un unexpected signer
$!
$!   >20 (bad)  problem not directly related to pgp analysis of signature
$!
$! By default public keyring and PGP configuration files
$! are expected in the directory NEWS_MANAGER_DEV:[PGP] .
$! To use other directory define logical name NEWS_PGPPATH .
$! Make sure this directory contains the file PUBRING.PGP
$! with the relevant public keys.  Up to date public keys
$! for the main hierarchies can be found at:
$!    ftp://ftp.isc.org/pub/pgpcontrol/PGPKEYS
$! See also:
$!    ftp://ftp.isc.org/pub/pgpcontrol/README
$! or ftp://ftp.isc.org/pub/pgpcontrol/README.html
$
$
$! V6.2.0 25-Mar-1997  Mark Martinec  mark.martinec@ijs.si
$! - initial version, roughly based on the last part of the
$!   perl script 'pgpverify' by David C Lawrence <tale@uunet.uu.net>,
$!   ftp://ftp.uu.net/networking/news/misc/pgpcontrol/pgpverify 
$! V6.2.0 23-May-1997  Mark Martinec  mark.martinec@ijs.si
$! - polishing, check environment, documentation, catchall
$!
$!
$
$	ok = 255
$	SS$_NORMAL = 1
$	exit_sts = %x10000000	! severity WARNING, don't print message
$	signer = ""
$	key_id = ""
$	hierarchy = ""
$	expected_signer = ""
$	say = "write sys$output"
$	on error     then goto err
$	on control_y then goto err
$
$!
$!------------ beginning of user-configurable part
$
$! Adjust to match PGP program location:
$ pgp_prog = "SYS$SYSTEM:PGP.EXE"
$
$
$! List of PGP key IDs of the control message signatorys per hierarchy:
$!   (check ftp://ftp.isc.org/pub/pgpcontrol/README for the up-to-date list)
$!
$! Symbol values may be of the form: "key_id signer_id" or just "signer_id".
$! In the first case the 8-digit hex key id is checked as well,
$! in the second case it is ignored.
$
$ pgpkeyid_comp		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_misc		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_news		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_rec		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_sci		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_soc		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_talk		= "B88DA9C1 news.announce.newgroups"
$ pgpkeyid_humanities	= "B88DA9C1 news.announce.newgroups"
$
$ pgpkeyid_gov		= "7FFD7855 gov.usenet.announce"
$ pgpkeyid_bit		= "06DBA755 bit.admin"
$ pgpkeyid_clari	= "A22FDBCD ClariNet.Group"
$
$ pgpkeyid_be		= "BA8601A1 be.announce.newgroups"
$ pgpkeyid_de		= "D3033C99 de.admin.news.announce"
$ pgpkeyid_es		= "6D457F19 es.news"
$ pgpkeyid_fr		= "382B64FD fr.announce.newgroups"
$ pgpkeyid_it		= "185317B9 it.announce.newgroups"
$ pgpkeyid_si		= "434E1955 si.news.announce.newsgroups"
$ pgpkeyid_uk		= "21F2F1D9 uk.net.news.announce"
$
$!  A little trick here. If a signatory for a particular hierarchy is
$!  not defined above, no signature is expected and as far as the
$!  signature verification is concerned the control message is valid
$!  and may be acted upon. If it is desired that the default action
$!  for hierarchies with no signatory is not to execute them (and mail
$!  the control message to "USENET" instead), one can define symbols like:
$!pgpkeyid_alt		= "00000000 nosuch"
$!
$!  On top of that it is possible to specify a default signatory
$!  (usually nonexistent, but possibly some campus-wide Usenet administrator)
$!  by defining a symbol "pgpkeyid_catchall".
$!
$!  To sum up: in order to reject (not execute) control messages pertaining to
$!  newsgroup hierarchies not defined above, uncomment the line below:
$!pgpkeyid_catchall	= "00000000 nobody"
$
$!  By doing so it is quite safe and useful to specify the qualifier /EXECUTE
$!  to the ADD FILE command.
$
$!------------ end of user-configurable part
$
$
$	pgp_prog = f$parse(pgp_prog,"SYS$SYSTEM:PGP.EXE")
$	if f$sear(pgp_prog).eqs."" then pgp_prog = "NEWS_MANAGER:PGP.EXE"
$	pgp_cmd  = "$" + pgp_prog
$	pgp_path = "NEWS_MANAGER_DEV:[PGP]"
$	if f$log("NEWS_PGPPATH").nes."" then pgp_path = "NEWS_PGPPATH:"
$
$	me = f$env("PROCEDURE") - f$parse(f$env("PROCEDURE"),,,"VERSION")
$	call check_file "''me'"
$	if .not.$status then goto exit
$	call check_file "''pgp_prog'"
$	if .not.$status then goto exit
$	call check_file "''pgp_path'PUBRING.PGP"
$	if .not.$status then goto exit
$
$	p1 = f$edit(p1,"TRIM")
$	if p2.eqs.""
$	then
$	  say "255 Usage: @", f$parse(f$env("PROCEDURE"),,,"NAME"), " filename newsgroup"
$	  goto exit
$	endif
$
$	hierarchy = f$edit(f$elem(0,".",p2),"LOWERCASE,TRIM")
$	allowed_chset = "abcdefghijklmnopqrstuvwxyz"
$	bad_syntax = 0
$	j = 0
$1:	if j.ge.f$len(hierarchy) then goto 2
$	  c = f$extr(j,1,hierarchy)
$	  if f$loc(f$extr(j,1,hierarchy), allowed_chset) -
             .eq. f$len(allowed_chset) then bad_syntax = 1
$	  allowed_chset = "abcdefghijklmnopqrstuvwxyz0123456789-"
$	  j = j+1
$	  goto 1
$2:	if hierarchy.eqs."" .or. bad_syntax
$	then
$	  say "255 Illegal hierarchy name: ", p2
$	  goto exit
$	endif
$
$	symbol = "pgpkeyid_" + hierarchy
$	if f$type('symbol').eqs."STRING"	! symbol defined?
$	then
$	  expected_signer = 'symbol'		! eval
$	else					! try if catchall exists
$	  symbol = "pgpkeyid_catchall"
$	  if f$type('symbol').eqs."STRING" then expected_signer = 'symbol'
$	endif
$
$	if p1.nes.""
$	then
$	  p1 = f$parse(p1,".ASC")		! default file type is ".ASC"
$	  if f$sear(p1).eqs.""
$	  then
$	    say "255 File ", p1, " not found"
$	    goto exit
$	  endif
$	endif
$
$	if p1.eqs.""
$	then
$	  ok = 1  ! no signature found
$	  exit_sts = SS$_NORMAL
$	  goto finish
$	else
$	  pid = f$getjpi("","PID")
$	  tmp_fn = f$parse("0PGPLOG"+pid, ".TMP;")
$	  if f$log("SYS$SCRATCH").nes."" then -
	    tmp_fn = f$parse(tmp_fn, "SYS$SCRATCH:")
$
$	  define/user/nolog PGPPATH 'pgp_path'
$	  define/user sys$input 'p1'
$	  define/user sys$output NL:
$	  define/user sys$error 'tmp_fn'
$	  set noon
$	  pgp_cmd -f
$	  if .not.$status
$	  then
$	    exit_sts = $status .and. %X0FFFFFFF
$	    set on
$	    goto finish
$	  endif
$	  set on
$
$	  has_sig = 0
$	  open/read f 'tmp_fn'
$rd:	  read/err=err/end=eof f line 
$	    patt = "File has signature"
$	    if f$extr(0,f$length(patt),line).eqs.patt then has_sig = 1
$	    patt = "Good signature from user"
$	    if f$extr(0,f$length(patt),line).eqs.patt then goto good
$	    patt = "Bad signature"
$	    if f$extr(0,f$length(patt),line).eqs.patt then goto bad
$	    goto rd
$eof:	  close f
$	  ok = 1  ! no signature found
$	  if has_sig then ok = 2  ! unknown signature
$	  exit_sts = SS$_NORMAL
$	  goto finish
$good:	  ok = 0
$	  exit_sts = SS$_NORMAL
$	  goto extract_signer
$bad:	  ok = 3  ! bad signature
$	  exit_sts = SS$_NORMAL
$	  patt = "Bad signature from user"
$	  if f$extr(0,f$length(patt),line).eqs.patt then goto extract_signer
$	  goto finish
$extract_signer:
$	  signer = f$extr(f$length(patt),f$len(line)-f$length(patt),line)
$	  if f$extr(0,1,signer).eqs.":" then signer = f$extr(1,f$len(signer)-1,signer)
$	  if f$extr(0,1,signer).eqs." " then signer = f$extr(1,f$len(signer)-1,signer)
$	  if f$extr(0,1,signer).eqs." " then signer = f$extr(1,f$len(signer)-1,signer)
$	  if f$extr(0,1,signer).eqs.""""
$	  then
$	    signer = f$extr(1,f$len(signer)-1,signer)
$	    if f$extr(f$len(signer)-1,1,signer).eqs."."  then signer = f$extr(0,f$len(signer)-1,signer)
$	    if f$extr(f$len(signer)-1,1,signer).eqs."""" then signer = f$extr(0,f$len(signer)-1,signer)
$	  endif
$	  read/err=err/end=e1 f line 
$	  patt = "Signature made"
$	  patt2 = "key ID "
$	  if f$extr(0,f$length(patt),line).eqs.patt .and. -
             f$loc(patt2,line).lt.f$len(line) then -
	    key_id = f$extr(f$loc(patt2,line)+f$len(patt2),1024,line)
$e1:	  if key_id.eqs."" then key_id = "00000000"
$	endif
$	goto finish
$
$err:	exit_sts = %x10000002	! severity ERROR, don't print message
$	goto exit
$
$finish:
$	if expected_signer.nes.""
$	then
$	  if ok.eq.0 .and. -
	     ((key_id+" "+signer).nes.expected_signer .and. -
	      signer.nes.expected_signer) then ok = 4
$	else
$	  if ok.eq.0 then ok = 4
$	  if ok.ge.1 .and. ok.le.4 then ok = ok + 10
$	endif
$	say f$fao("!03ZL !AS !AS", ok,key_id,signer)
$
$exit:	if f$log("f").nes."" then close f
$	if f$type(tmp_fn).eqs."STRING"
$	then
	  if f$sear(tmp_fn).nes."" then -
	    delete/noconfirm/nolog/erase 'f$parse(";*",tmp_fn)'
$	endif
$	exit 'exit_sts + 0*f$ver(ver)'
$
$check_file:
$	subroutine
$	  bad_status = %x10000000	! severity WARNING, don't print message
$	  if f$sear(p1).eqs.""
$	  then
$	    say "255 File ", p1, " does not exist, quitting"
$	    exit 'bad_status'
$	  endif
$	  prot_w_field = f$elem(1,"=",f$elem(3,",",f$file(p1,"PRO")))
$	  prot_g_field = f$elem(1,"=",f$elem(2,",",f$file(p1,"PRO")))
$	  world_writable = f$loc("W",prot_w_field).lt.f$len(prot_w_field)
$	  group_writable = f$loc("W",prot_g_field).lt.f$len(prot_g_field)
$	  if world_writable
$	  then
$	    say "255 File ", p1, " is world writable, quitting"
$	    exit 'bad_status'
$	  endif
$	  if group_writable
$	  then
$	    say "255 File ", p1, " is group writable, quitting"
$	    exit 'bad_status'
$	  endif
$	  exit 1
$	endsubroutine
